@@ -3,6 +3,7 @@
 from __future__ import annotations
 
 from collections.abc import Awaitable, Callable
+from dataclasses import dataclass
 from typing import Any, Literal
 
 from acp.schema import (
@@ -62,6 +63,43 @@ StopReason = Literal[
 # Plan entry types
 PlanEntryPriority = Literal["high", "medium", "low"]
 PlanEntryStatus = Literal["pending", "in_progress", "completed"]
-
+ToolCallKind = Literal[
+    "read",
+    "edit",
+    "delete",
+    "move",
+    "search",
+    "execute",
+    "think",
+    "fetch",
+    "switch_mode",
+    "other",
+]
 JsonValue = Any
 MethodHandler = Callable[[str, JsonValue | None, bool], Awaitable[JsonValue | None]]
+
+
+@dataclass
+class SessionData:
+    """Complete session data for persistence and restoration."""
+
+    session_id: str
+    """Unique session identifier"""
+
+    model_name: str
+    """Currently chosen model"""
+
+    session_mode: str | None
+    """Currently selected session mode"""
+
+    cwd: str
+    """Working directory for the session"""
+
+    mcp_servers: list[MCPServer]
+    """MCP server configurations"""
+
+    notifications: list[SessionUpdate]
+    """Complete notification history"""
+
+    metadata: dict[str, Any]
+    """Additional session metadata"""
