@@ -11,13 +11,14 @@ import time
 from typing import TYPE_CHECKING, Any, Literal, Self, TypedDict, TypeVar, cast, overload
 from uuid import uuid4
 
+from anyenv import method_spawner
 from llmling import Config, RuntimeConfig, ToolError
+import logfire
 from psygnal import Signal
 
 from llmling_agent.log import get_logger
 from llmling_agent.messaging.messagenode import MessageNode
 from llmling_agent.messaging.messages import ChatMessage, TokenCost
-from llmling_agent.observability import track_action
 from llmling_agent.prompts.builtin_provider import RuntimePromptProvider
 from llmling_agent.prompts.convert import convert_prompts
 from llmling_agent.resource_providers.runtime import RuntimeResourceProvider
@@ -27,11 +28,10 @@ from llmling_agent.tools.manager import ToolManager
 from llmling_agent.utils.inspection import (
     call_with_context,
     has_return_type,
-    validate_import,
 )
 from llmling_agent.utils.now import get_now
 from llmling_agent.utils.result_utils import to_type
-from llmling_agent.utils.tasks import TaskManagerMixin
+from llmling_agent.utils.tasks import TaskManager
 from llmling_agent_config.session import MemoryConfig
 
 
@@ -44,7 +44,8 @@ if TYPE_CHECKING:
     from llmling.config.models import Resource
     from llmling.prompts import PromptType
     import PIL.Image
-    from pydantic_ai.run import AgentRun
+    from pydantic_ai.agent import EventStreamHandler
+    from pydantic_ai.messages import AgentStreamEvent
     from toprompt import AnyPromptType
 
     from llmling_agent.agent import AgentContext, AnyAgent
@@ -68,7 +69,7 @@ if TYPE_CHECKING:
     from llmling_agent_config.session import SessionQuery
     from llmling_agent_config.task import Job
     from llmling_agent_input.base import InputProvider
-    from llmling_agent_providers.base import StreamingResponseProtocol, UsageLimits
+    from llmling_agent_providers.base import UsageLimits
 
 from llmling_agent_providers.base import AgentProvider
 
@@ -80,6 +81,16 @@ logger = get_logger(__name__)
 TResult = TypeVar("TResult", default=str)
 
 
+@dataclass(repr=False, kw_only=True)
+class StreamCompleteEvent:
+    """Event indicating streaming is complete with final message."""
+
+    message: ChatMessage[Any]
+    """The final chat message with all metadata."""
+    event_kind: Literal["stream_complete"] = "stream_complete"
+    """Event type identifier."""
+
+
 class AgentKwargs(TypedDict, total=False):
     """Keyword arguments for configuring an Agent instance."""
 
@@ -111,9 +122,10 @@ class AgentKwargs(TypedDict, total=False):
     # Behavior Control
     input_provider: InputProvider | None
     debug: bool
+    event_handlers: Sequence[EventStreamHandler] | None
 
 
-class Agent[TDeps = None](MessageNode[TDeps, str], TaskManagerMixin):
+class Agent[TDeps = None](MessageNode[TDeps, str]):
     """Agent for AI-powered interaction with LLMling resources and tools.
 
     Generically typed with: LLMLingAgent[Type of Dependencies, Type of Result]
@@ -168,6 +180,7 @@ class Agent[TDeps = None](MessageNode[TDeps, str], TaskManagerMixin):
         input_provider: InputProvider | None = None,
         parallel_init: bool = True,
         debug: bool = False,
+        event_handlers: Sequence[EventStreamHandler] | None = None,
     ):
         """Initialize agent with runtime configuration.
 
@@ -198,6 +211,7 @@ class Agent[TDeps = None](MessageNode[TDeps, str], TaskManagerMixin):
             input_provider: Provider for human input (tool confirmation / HumanProviders)
             parallel_init: Whether to initialize resources in parallel
             debug: Whether to enable debug mode
+            event_handlers: Sequence of event handlers to register with the agent
         """
         from llmling_agent.agent import AgentContext
         from llmling_agent.agent.conversation import ConversationManager
@@ -208,6 +222,7 @@ class Agent[TDeps = None](MessageNode[TDeps, str], TaskManagerMixin):
         )
         from llmling_agent_providers.base import AgentProvider
 
+        self.task_manager = TaskManager()
         self._infinite = False
         # save some stuff for asnyc init
         self._owns_runtime = False
@@ -245,6 +260,7 @@ class Agent[TDeps = None](MessageNode[TDeps, str], TaskManagerMixin):
         runtime_provider = RuntimePromptProvider(ctx.runtime)
         ctx.definition.prompt_manager.providers["runtime"] = runtime_provider
         # Initialize tool manager
+        self._event_handlers = event_handlers or []
         all_tools = list(tools or [])
         self.tools = ToolManager(all_tools)
         self.tools.add_provider(self.mcp)
@@ -259,7 +275,6 @@ class Agent[TDeps = None](MessageNode[TDeps, str], TaskManagerMixin):
         # Initialize provider
         match provider:
             case "pydantic_ai":
-                validate_import("pydantic_ai", "pydantic_ai")
                 from llmling_agent_providers.pydanticai import PydanticAIProvider
 
                 if model and not isinstance(model, str):
@@ -296,7 +311,7 @@ class Agent[TDeps = None](MessageNode[TDeps, str], TaskManagerMixin):
         if ctx and ctx.definition:
             from llmling_agent.observability import registry
 
-            registry.register_providers(ctx.definition.observability)
+            registry.configure_observability(ctx.definition.observability)
 
         # init variables
         self._debug = debug
@@ -306,7 +321,6 @@ class Agent[TDeps = None](MessageNode[TDeps, str], TaskManagerMixin):
         self._background_task: asyncio.Task[Any] | None = None
 
         # Forward provider signals
-        self._provider.chunk_streamed.connect(self.chunk_streamed)
         self._provider.model_changed.connect(self.model_changed)
         self._provider.tool_used.connect(self.tool_used)
         self._provider.model_changed.connect(self.model_changed)
@@ -491,6 +505,15 @@ class Agent[TDeps = None](MessageNode[TDeps, str], TaskManagerMixin):
         self._provider.name = value
         self._name = value
 
+    @property
+    def event_handler(self) -> EventStreamHandler:
+        async def merged(*args: Any, **kwargs: Any) -> Any:
+            return await asyncio.gather(*[
+                h(*args, **kwargs) for h in self._event_handlers
+            ])
+
+        return merged
+
     @property
     def context(self) -> AgentContext[TDeps]:
         """Get agent context."""
@@ -536,7 +559,6 @@ class Agent[TDeps = None](MessageNode[TDeps, str], TaskManagerMixin):
 
         name = self.name
         debug = self._debug
-        self._provider.chunk_streamed.disconnect(self.chunk_streamed)
         self._provider.model_changed.disconnect(self.model_changed)
         self._provider.tool_used.disconnect(self.tool_used)
         self._provider.model_changed.disconnect(self.model_changed)
@@ -544,7 +566,6 @@ class Agent[TDeps = None](MessageNode[TDeps, str], TaskManagerMixin):
             case AgentProvider():
                 self._provider = value
             case "pydantic_ai":
-                validate_import("pydantic_ai", "pydantic_ai")
                 from llmling_agent_providers.pydanticai import PydanticAIProvider
 
                 self._provider = PydanticAIProvider(model=model, name=name, debug=debug)
@@ -559,7 +580,6 @@ class Agent[TDeps = None](MessageNode[TDeps, str], TaskManagerMixin):
             case _:
                 msg = f"Invalid agent type: {type}"
                 raise ValueError(msg)
-        self._provider.chunk_streamed.connect(self.chunk_streamed)
         self._provider.model_changed.connect(self.model_changed)
         self._provider.tool_used.connect(self.tool_used)
         self._provider.model_changed.connect(self.model_changed)
@@ -622,7 +642,7 @@ class Agent[TDeps = None](MessageNode[TDeps, str], TaskManagerMixin):
 
     def is_busy(self) -> bool:
         """Check if agent is currently processing tasks."""
-        return bool(self._pending_tasks or self._background_task)
+        return bool(self.task_manager._pending_tasks or self._background_task)
 
     @property
     def model_name(self) -> str | None:
@@ -681,7 +701,7 @@ class Agent[TDeps = None](MessageNode[TDeps, str], TaskManagerMixin):
             description_override=docstring,
         )
 
-    @track_action("Calling Agent.run: {prompts}:")
+    @logfire.instrument("Calling Agent.run: {prompts}:")
     async def _run(
         self,
         *prompts: AnyPromptType | PIL.Image.Image | os.PathLike[str] | ChatMessage[Any],
@@ -761,7 +781,6 @@ class Agent[TDeps = None](MessageNode[TDeps, str], TaskManagerMixin):
                 devtools.debug(response_msg)
             return response_msg
 
-    @asynccontextmanager
     async def run_stream(
         self,
         *prompt: AnyPromptType | PIL.Image.Image | os.PathLike[str],
@@ -774,7 +793,7 @@ class Agent[TDeps = None](MessageNode[TDeps, str], TaskManagerMixin):
         conversation_id: str | None = None,
         messages: list[ChatMessage[Any]] | None = None,
         wait_for_connections: bool | None = None,
-    ) -> AsyncIterator[StreamingResponseProtocol[TResult]]:
+    ) -> AsyncIterator[AgentStreamEvent | StreamCompleteEvent]:
         """Run agent with prompt and get a streaming response.
 
         Args:
@@ -790,9 +809,8 @@ class Agent[TDeps = None](MessageNode[TDeps, str], TaskManagerMixin):
             conversation_id: Optional conversation id for the returned message.
             messages: Optional list of messages to replace the conversation history
             wait_for_connections: Whether to wait for connected agents to complete
-
         Returns:
-            A streaming result to iterate over.
+            An async iterator yielding streaming events with final message embedded.
 
         Raises:
             UnexpectedModelBehavior: If the model fails or behaves unexpectedly
@@ -807,41 +825,65 @@ class Agent[TDeps = None](MessageNode[TDeps, str], TaskManagerMixin):
             messages if messages is not None else self.conversation.get_history()
         )
         try:
-            async with self._provider.stream_response(
+            # Collect chunks for final message construction
+            chunks = []
+            usage = None
+            model_name = None
+
+            # Stream events directly from provider
+            async for event in self._provider.stream_events(
                 *prompts,
                 message_id=message_id,
                 message_history=message_history,
                 result_type=result_type,
                 model=model,
-                store_history=store_history,
                 tools=tools,
                 usage_limits=usage_limits,
                 system_prompt=sys_prompt,
-            ) as stream:
-                yield stream
-                usage = stream.usage()
-                cost_info = None
-                model_name = stream.model_name  # type: ignore
-                if model_name:
-                    cost_info = await TokenCost.from_usage(usage, model_name)
-                response_msg = ChatMessage[TResult](
-                    content=cast(TResult, stream.formatted_content),  # type: ignore
-                    role="assistant",
-                    name=self.name,
-                    model=model_name,
-                    message_id=message_id,
-                    conversation_id=user_msg.conversation_id,
-                    cost_info=cost_info,
-                    response_time=time.perf_counter() - start_time,
-                    # provider_extra=stream.provider_extra or {},
-                )
-                self.message_sent.emit(response_msg)
-                if store_history:
-                    self.conversation.add_chat_messages([user_msg, response_msg])
-                await self.connections.route_message(
-                    response_msg,
-                    wait=wait_for_connections,
-                )
+            ):
+                from pydantic_ai.messages import PartDeltaEvent, TextPartDelta
+                from pydantic_ai.run import AgentRunResultEvent
+
+                # Pass through PydanticAI events and collect chunks
+                match event:
+                    case PartDeltaEvent(delta=TextPartDelta(content_delta=delta)):
+                        chunks.append(delta)
+                        self.chunk_streamed.emit(delta, message_id)
+                        yield event  # Pass through original event
+                    case AgentRunResultEvent(result=result):
+                        usage = result.usage()
+                        model_name = result.response.model_name
+                        # Don't yield AgentRunResultEvent, we'll send our own final event
+                    case _:
+                        yield event  # Pass through other events
+
+            # Build final chat message
+            cost_info = None
+            if model_name and usage and model_name != "test":
+                cost_info = await TokenCost.from_usage(usage, model_name)
+
+            response_msg = ChatMessage[TResult](
+                content=cast(TResult, "".join(chunks)),  # type: ignore
+                role="assistant",
+                name=self.name,
+                model=model_name,
+                message_id=message_id,
+                conversation_id=user_msg.conversation_id,
+                cost_info=cost_info,
+                response_time=time.perf_counter() - start_time,
+            )
+
+            # Yield final event with embedded message
+            yield StreamCompleteEvent(message=response_msg)
+
+            # Post-processing
+            self.message_sent.emit(response_msg)
+            if store_history:
+                self.conversation.add_chat_messages([user_msg, response_msg])
+            await self.connections.route_message(
+                response_msg,
+                wait=wait_for_connections,
+            )
 
         except Exception as e:
             logger.exception("Agent stream failed")
@@ -887,34 +929,7 @@ class Agent[TDeps = None](MessageNode[TDeps, str], TaskManagerMixin):
             )
             yield response  # pyright: ignore
 
-    def run_sync(
-        self,
-        *prompt: AnyPromptType | PIL.Image.Image | os.PathLike[str],
-        result_type: type[TResult] | None = None,
-        deps: TDeps | None = None,
-        model: ModelType = None,
-        store_history: bool = True,
-    ) -> ChatMessage[TResult]:
-        """Run agent synchronously (convenience wrapper).
-
-        Args:
-            prompt: User query or instruction
-            result_type: Optional type for structured responses
-            deps: Optional dependencies for the agent
-            model: Optional model override
-            store_history: Whether the message exchange should be added to the
-                           context window
-        Returns:
-            Result containing response and run information
-        """
-        coro = self.run(
-            *prompt,
-            model=model,
-            store_history=store_history,
-            result_type=result_type,
-        )
-        return self.run_task_sync(coro)  # type: ignore
-
+    @method_spawner
     async def run_job(
         self,
         job: Job[TDeps, str | None],
@@ -968,115 +983,6 @@ class Agent[TDeps = None](MessageNode[TDeps, str], TaskManagerMixin):
             logger.exception(msg)
             raise JobError(msg) from e
 
-    @asynccontextmanager
-    @track_action("Calling Agent.iterate_run: {prompts}")
-    async def iterate_run(
-        self,
-        *prompts: AnyPromptType | PIL.Image.Image | os.PathLike[str],
-        message_id: str | None = None,
-        message_history: list[ChatMessage[Any]] | None = None,
-        tools: list[Tool] | None = None,
-        result_type: type[TResult] | None = None,
-        usage_limits: UsageLimits | None = None,
-        model: ModelType = None,
-        system_prompt: str | None = None,
-        tool_choice: str | list[str] | None = None,
-        conversation_id: str | None = None,
-        store_history: bool = True,
-    ) -> AsyncIterator[AgentRun[TDeps, TResult]]:
-        """Run the agent step-by-step, yielding an object to observe the execution graph.
-
-        Args:
-            *prompts: User query/instructions (text, images, paths, BasePrompts).
-            message_id: Optional unique ID for this run attempt. Generated if None.
-            message_history: Optional list of messages to replace current history.
-            tools: Optional sequence of tools to use instead of agent's default tools.
-            result_type: Optional type for structured responses.
-            usage_limits: Optional usage limits (provider support may vary).
-            model: Optional model override for this run.
-            system_prompt: Optional system prompt override for this run.
-            tool_choice: Filter agent's tools by name (ignored if `tools` is provided).
-            conversation_id: Optional ID to associate with the conversation context.
-            store_history: Whether to store the conversation in agent's history.
-
-        Yields:
-            An AgentRun object for iterating over execution nodes.
-
-        Example: (Same as before)
-            async with agent.iterate_run("Capital of France?") as agent_run:
-                async for node in agent_run:
-                    print(f"Processing: {type(node).__name__}")
-                print(f"Final result: {agent_run.result.output}")
-
-        Note: (Same as before regarding history management)
-        """
-        run_message_id = message_id or str(uuid4())
-        start_time = time.perf_counter()
-        logger.info("Starting agent iteration run_id=%s", run_message_id)
-        converted_prompts = await convert_prompts(prompts)
-        if not converted_prompts:
-            msg = "No prompts provided for iteration."
-            logger.error(msg)
-            raise ValueError(msg)
-
-        # Prepare user message for conversation history
-        user_msg = None
-        if store_history:
-            user_msg, _ = await self.pre_run(*prompts)
-
-        if tools is None:
-            effective_tools = await self.tools.get_tools(
-                state="enabled", names=tool_choice
-            )
-        else:
-            effective_tools = tools  # Use the direct override
-
-        self.set_result_type(result_type)
-        effective_system_prompt = (
-            system_prompt
-            if system_prompt is not None
-            else await self.sys_prompts.format_system_prompt(self)
-        )
-        effective_message_history = (
-            message_history
-            if message_history is not None
-            else self.conversation.get_history()
-        )
-        try:
-            async with self._provider.iterate_run(
-                *converted_prompts,
-                # Pass consistent arguments to the provider
-                message_id=run_message_id,
-                message_history=effective_message_history,
-                tools=effective_tools,
-                result_type=result_type,
-                usage_limits=usage_limits,
-                model=model,
-                system_prompt=effective_system_prompt,
-            ) as agent_run:
-                yield agent_run
-                # Store conversation history if requested
-                if store_history and user_msg and agent_run.result:
-                    response_msg = ChatMessage[TResult](
-                        content=agent_run.result.output,
-                        role="assistant",
-                        name=self.name,
-                        model=getattr(agent_run.result, "model_name", None),
-                        message_id=run_message_id,
-                        conversation_id=conversation_id or user_msg.conversation_id,
-                        response_time=time.perf_counter() - start_time,
-                    )
-                    self.conversation.add_chat_messages([user_msg, response_msg])
-                    msg = "Stored conversation history for run_id=%s"
-                    logger.debug(msg, run_message_id)
-
-            logger.info("Agent iteration run_id=%s completed.", run_message_id)
-
-        except Exception as e:
-            logger.exception("Agent iteration run_id=%s failed.", run_message_id)
-            self.run_failed.emit(f"Agent iteration failed: {e}", e)
-            raise
-
     async def run_in_background(
         self,
         *prompt: AnyPromptType | PIL.Image.Image | os.PathLike[str],
@@ -1355,16 +1261,18 @@ class Agent[TDeps = None](MessageNode[TDeps, str], TaskManagerMixin):
 
 
 if __name__ == "__main__":
-    # import logging
+    import logging
 
+    logfire.configure()
+    logfire.instrument_pydantic_ai()
+    logging.basicConfig(handlers=[logfire.LogfireLoggingHandler()])
     sys_prompt = "Open browser with google,"
     _model = "openai:gpt-5-nano"
 
     async def main():
         async with Agent[None](model=_model, tools=["webbrowser.open"]) as agent:
             agent.tool_used.connect(print)
-            async with agent.iterate_run(sys_prompt) as stream:
-                async for chunk in stream:
-                    print(chunk)
+            async for chunk in agent.run_stream(sys_prompt):
+                print(chunk)
 
     asyncio.run(main())
