@@ -7,6 +7,7 @@ from time import perf_counter
 from typing import TYPE_CHECKING, Any, TypeVar
 from uuid import uuid4
 
+from aioitertools.asyncio import as_generated
 from toprompt import to_prompt
 
 from llmling_agent.delegation.base_team import BaseTeam
@@ -22,9 +23,11 @@ if TYPE_CHECKING:
     import os
 
     import PIL.Image
+    from pydantic_ai.messages import AgentStreamEvent
     from toprompt import AnyPromptType
 
     from llmling_agent import MessageNode
+    from llmling_agent.agent.agent import StreamCompleteEvent
     from llmling_agent.talk import Talk
     from llmling_agent_config.task import Job
 
@@ -32,6 +35,36 @@ if TYPE_CHECKING:
 TDeps = TypeVar("TDeps", default=None)
 
 
+async def normalize_stream_for_teams(
+    node: MessageNode[Any, Any],
+    *args: Any,
+    **kwargs: Any,
+) -> AsyncIterator[tuple[MessageNode[Any, Any], AgentStreamEvent | StreamCompleteEvent]]:
+    """Normalize any streaming node to yield (node, event) tuples for team composition.
+
+    Args:
+        node: The streaming node (Agent, Team, etc.)
+        *args: Arguments to pass to run_stream
+        **kwargs: Keyword arguments to pass to run_stream
+
+    Yields:
+        Tuples of (node, event) where node is the MessageNode instance
+        and event is the streaming event from that node.
+    """
+    if not hasattr(node, "run_stream"):
+        msg = f"Node {node.name} does not support streaming"
+        raise ValueError(msg)
+
+    stream = node.run_stream(*args, **kwargs)  # type: ignore[attr-defined]
+    async for item in stream:
+        if isinstance(item, tuple):
+            # Already normalized (from Team or other composite node)
+            yield item
+        else:
+            # Raw event (from Agent) - wrap it with the source node
+            yield (node, item)
+
+
 class Team[TDeps](BaseTeam[TDeps, Any]):
     """Group of agents that can execute together."""
 
@@ -162,6 +195,38 @@ class Team[TDeps](BaseTeam[TDeps, Any]):
             },
         )
 
+    async def run_stream(
+        self,
+        *prompts: AnyPromptType | PIL.Image.Image | os.PathLike[str],
+        **kwargs: Any,
+    ) -> AsyncIterator[
+        tuple[MessageNode[Any, Any], AgentStreamEvent | StreamCompleteEvent]
+    ]:
+        """Stream responses from all team members in parallel.
+
+        Args:
+            prompts: Input prompts to process in parallel
+            kwargs: Additional arguments passed to each agent
+
+        Yields:
+            Tuples of (agent, event) where agent is the Agent instance
+            and event is the streaming event from that agent.
+        """
+        # Get nodes to run
+        combined_prompt = "\n".join([await to_prompt(p) for p in prompts])
+        all_nodes = list(await self.pick_agents(combined_prompt))
+
+        # Create list of streams that yield (agent, event) tuples
+        agent_streams = [
+            normalize_stream_for_teams(agent, *prompts, **kwargs)
+            for agent in all_nodes
+            if hasattr(agent, "run_stream")
+        ]
+
+        # Merge all agent streams
+        async for agent_event_tuple in as_generated(agent_streams):
+            yield agent_event_tuple
+
     async def run_job[TJobResult](
         self,
         job: Job[TDeps, TJobResult],
@@ -238,3 +303,24 @@ class Team[TDeps](BaseTeam[TDeps, Any]):
             msg = "Job execution failed"
             logger.exception(msg)
             raise JobError(msg) from e
+
+
+if __name__ == "__main__":
+    from llmling_agent import Agent
+
+    async def main():
+        from llmling_agent.delegation.teamrun import TeamRun
+
+        agent_a = Agent(name="A", model="test")
+        agent_b = Agent(name="B", model="test")
+        agent_c = Agent(name="C", model="test")
+
+        # Test Team containing TeamRun (parallel containing sequential)
+        inner_run = TeamRun([agent_a, agent_b], name="Sequential")
+        outer_team = Team([inner_run, agent_c], name="Parallel")
+
+        print("Testing Team containing TeamRun...")
+        async for node, event in outer_team.run_stream("test"):
+            print(f"{node.name}: {type(event).__name__}")
+
+    asyncio.run(main())
