@@ -3,10 +3,10 @@
 from __future__ import annotations
 
 from contextlib import AsyncExitStack, suppress
+import logging
+from pathlib import Path
 import shutil
-from typing import TYPE_CHECKING, Any, Self, TextIO
-
-import mcp
+from typing import TYPE_CHECKING, Any, Self
 
 from llmling_agent.log import get_logger
 
@@ -15,9 +15,15 @@ if TYPE_CHECKING:
     from collections.abc import Awaitable, Callable
     from types import TracebackType
 
+    import mcp
     from mcp import ClientSession
-    from mcp.client.session import RequestContext
-    from mcp.shared.session import ProgressFnT
+    from mcp.client.session import (
+        ElicitationFnT,
+        MessageHandlerFnT,
+        RequestContext,
+        SamplingFnT,
+    )
+    from mcp.shared.session import ProgressFnT, RequestResponder
     from mcp.types import Tool, Tool as MCPTool
     from pydantic_ai import RunContext
 
@@ -33,33 +39,76 @@ def mcp_tool_to_fn_schema(tool: MCPTool) -> dict[str, Any]:
     return {"name": tool.name, "description": desc, "parameters": tool.inputSchema}
 
 
+async def message_handler(
+    message: RequestResponder[mcp.ServerRequest, mcp.ClientResult]
+    | mcp.ServerNotification
+    | Exception,
+):
+    """Handle MCP server messages."""
+    import mcp
+    from mcp.shared.session import RequestResponder
+    from mcp.types import ClientResult, EmptyResult
+
+    if isinstance(message, Exception):
+        raise message
+    if isinstance(message, RequestResponder):
+        with message as resp:
+            await resp.respond(ClientResult(root=EmptyResult()))
+            return
+    match message.root:
+        case mcp.types.CancelledNotification(params=params):
+            logger.info("MCP operation cancelled: %s", params)
+        case mcp.ProgressNotification(params=params):
+            logger.debug("MCP progress: %s", params)
+        case mcp.LoggingMessageNotification(params=params):
+            # Map MCP log levels to Python logging integer levels
+            level_map = {
+                "debug": logging.DEBUG,
+                "info": logging.INFO,
+                "notice": logging.INFO,
+                "warning": logging.WARNING,
+                "error": logging.ERROR,
+                "critical": logging.CRITICAL,
+                "alert": logging.CRITICAL,
+                "emergency": logging.CRITICAL,
+            }
+
+            log_level = level_map[params.level]
+            logger_name = f"mcp.{params.logger}" if params.logger else "mcp.server"
+            mcp_logger = get_logger(logger_name)
+            log_msg = str(params.data) if params.data else "MCP server log message"
+            mcp_logger.log(log_level, log_msg)
+
+        case mcp.ResourceUpdatedNotification(uri=uri):
+            logger.info("MCP resource updated: %s", uri)
+        case mcp.types.ResourceListChangedNotification(params=params):
+            logger.info("MCP resource list changed: %s", params)
+        case mcp.types.ToolListChangedNotification(params=params):
+            logger.info("MCP tool list changed: %s", params)
+        case mcp.types.PromptListChangedNotification(params=params):
+            logger.info("MCP prompt list changed: %s", params)
+
+
 class MCPClient:
     """MCP client for communicating with MCP servers."""
 
     def __init__(
         self,
         transport_mode: TransportType = "stdio",
-        elicitation_callback: Callable[
-            [RequestContext, mcp.types.ElicitRequestParams],
-            Awaitable[mcp.types.ElicitResult | mcp.types.ErrorData],
-        ]
-        | None = None,
-        sampling_callback: Callable[
-            [RequestContext, mcp.types.CreateMessageRequestParams],
-            Awaitable[mcp.types.CreateMessageResult | mcp.types.ErrorData],
-        ]
-        | None = None,
+        elicitation_callback: ElicitationFnT | None = None,
+        sampling_callback: SamplingFnT | None = None,
         progress_handler: ProgressHandler | None = None,
+        message_handler: MessageHandlerFnT | None = None,
         accessible_roots: list[str] | None = None,
     ):
         self.exit_stack = AsyncExitStack()
         self.session: ClientSession | None = None
         self._available_tools: list[Tool] = []
-        self._old_stdout: TextIO | None = None
         self._transport_mode = transport_mode
         self._elicitation_callback = elicitation_callback
         self._sampling_callback = sampling_callback
         self._progress_handler = progress_handler
+        self._message_handler = message_handler
         self._accessible_roots = accessible_roots or []
 
     async def __aenter__(self) -> Self:
@@ -160,7 +209,7 @@ class MCPClient:
             context: RequestContext,
         ) -> mcp.types.ListRootsResult | mcp.types.ErrorData:
             """List accessible filesystem roots."""
-            from pathlib import Path
+            import mcp
 
             roots = []
             for root_path in self._accessible_roots:
@@ -185,6 +234,7 @@ class MCPClient:
             elicitation_callback=elicitation_wrapper,
             sampling_callback=sampling_wrapper,
             list_roots_callback=list_roots_wrapper,
+            message_handler=self._message_handler,
         )
         self.session = await self.exit_stack.enter_async_context(session)
         assert self.session
