@@ -2,6 +2,7 @@
 
 from __future__ import annotations
 
+import asyncio
 from collections.abc import Awaitable, Callable
 from dataclasses import dataclass
 from functools import cached_property
@@ -40,8 +41,8 @@ class NodeContext[TDeps]:
     current_prompt: str | None = None
     """Current prompt text for the agent."""
 
-    in_async_context: bool = False
-    """Whether we're running in an async context."""
+    # in_async_context: bool = False
+    # """Whether we're running in an async context."""
 
     input_provider: InputProvider | None = None
     """Provider for human-input-handling."""
@@ -79,6 +80,12 @@ class NodeContext[TDeps]:
     @property
     def report_progress(self) -> ProgressCallback | None:
         """Access progress reporting from pool server if available."""
-        return (
-            self.pool.server.report_progress if self.pool and self.pool.server else None
-        )
+
+        async def bundled_progress(current: float, total: float | None = None) -> None:
+            """Bundle multiple progress callbacks into one."""
+            if self.pool and self.pool.progress_handlers:
+                await asyncio.gather(
+                    *(handler(current, total) for handler in self.pool.progress_handlers)
+                )
+
+        return bundled_progress if self.pool and self.pool.progress_handlers else None
