@@ -2,6 +2,8 @@
 
 from __future__ import annotations
 
+from collections.abc import Sequence  # noqa: TC003
+from pathlib import Path
 from typing import TYPE_CHECKING, Any, Literal
 from uuid import UUID
 
@@ -18,6 +20,7 @@ from llmling.utils.importing import import_callable, import_class
 from pydantic import Field, model_validator
 from schemez import InlineSchemaDef
 from toprompt import render_prompt
+from upath import UPath
 
 from llmling_agent import log
 from llmling_agent.common_types import EndStrategy  # noqa: TC001
@@ -34,6 +37,7 @@ from llmling_agent_config.nodes import NodeConfig
 from llmling_agent_config.providers import ProviderConfig  # noqa: TC001
 from llmling_agent_config.result_types import StructuredResponseConfig  # noqa: TC001
 from llmling_agent_config.session import MemoryConfig, SessionQuery
+from llmling_agent_config.system_prompts import PromptConfig  # noqa: TC001
 from llmling_agent_config.tools import BaseToolConfig, ToolConfig  # noqa: TC001
 from llmling_agent_config.toolsets import ToolsetConfig  # noqa: TC001
 from llmling_agent_config.workers import WorkerConfig  # noqa: TC001
@@ -111,11 +115,8 @@ class AgentConfig(NodeConfig):
     avatar: str | None = None
     """URL or path to agent's avatar image"""
 
-    system_prompts: list[str] = Field(default_factory=list)
-    """System prompts for the agent"""
-
-    library_system_prompts: list[str] = Field(default_factory=list)
-    """System prompts for the agent from the library"""
+    system_prompts: Sequence[str | PromptConfig] = Field(default_factory=list)
+    """System prompts for the agent. Can be strings or structured prompt configs."""
 
     user_prompts: list[str] = Field(default_factory=list)
     """Default user prompts for the agent"""
@@ -176,10 +177,8 @@ class AgentConfig(NodeConfig):
     @classmethod
     def handle_model_types(cls, data: dict[str, Any]) -> dict[str, Any]:
         """Convert model inputs to appropriate format."""
-        model = data.get("model")
-        match model:
-            case str():
-                data["model"] = {"type": "string", "identifier": model}
+        if isinstance((model := data.get("model")), str):
+            data["model"] = {"type": "string", "identifier": model}
         return data
 
     async def get_toolsets(self) -> list[ResourceProvider]:
@@ -246,6 +245,13 @@ class AgentConfig(NodeConfig):
 
     def get_system_prompts(self) -> list[BasePrompt]:
         """Get all system prompts as BasePrompts."""
+        from llmling_agent_config.system_prompts import (
+            FilePromptConfig,
+            FunctionPromptConfig,
+            LibraryPromptConfig,
+            StaticPromptConfig,
+        )
+
         prompts: list[BasePrompt] = []
         for prompt in self.system_prompts:
             match prompt:
@@ -257,6 +263,49 @@ class AgentConfig(NodeConfig):
                         messages=[PromptMessage(role="system", content=prompt)],
                     )
                     prompts.append(static_prompt)
+                case StaticPromptConfig(content=content):
+                    # Convert StaticPromptConfig to StaticPrompt
+                    static_prompt = StaticPrompt(
+                        name="system",
+                        description="System prompt",
+                        messages=[PromptMessage(role="system", content=content)],
+                    )
+                    prompts.append(static_prompt)
+                case FilePromptConfig(path=path):
+                    # Load and convert file-based prompt
+
+                    template_path = Path(path)
+                    if not template_path.is_absolute() and self.config_file_path:
+                        base_path = Path(self.config_file_path).parent
+                        template_path = base_path / path
+
+                    template_content = template_path.read_text()
+                    # Create a template-based prompt
+                    # (for now as StaticPrompt with placeholder)
+                    static_prompt = StaticPrompt(
+                        name="system",
+                        description=f"File prompt: {path}",
+                        messages=[PromptMessage(role="system", content=template_content)],
+                    )
+                    prompts.append(static_prompt)
+                case LibraryPromptConfig(reference=reference):
+                    # Create placeholder for library prompts (resolved by manifest)
+                    msg = PromptMessage(role="system", content=f"[LIBRARY:{reference}]")
+                    static_prompt = StaticPrompt(
+                        name="system",
+                        description=f"Library: {reference}",
+                        messages=[msg],
+                    )
+                    prompts.append(static_prompt)
+                case FunctionPromptConfig(arguments=arguments, function=function):
+                    # Import and call the function to get prompt content
+                    content = function(**arguments)
+                    static_prompt = StaticPrompt(
+                        name="system",
+                        description=f"Function prompt: {function}",
+                        messages=[PromptMessage(role="system", content=content)],
+                    )
+                    prompts.append(static_prompt)
                 case BasePrompt():
                     prompts.append(prompt)
         return prompts
@@ -295,10 +344,43 @@ class AgentConfig(NodeConfig):
 
     def render_system_prompts(self, context: dict[str, Any] | None = None) -> list[str]:
         """Render system prompts with context."""
+        from llmling_agent_config.system_prompts import (
+            FilePromptConfig,
+            FunctionPromptConfig,
+            LibraryPromptConfig,
+            StaticPromptConfig,
+        )
+
         if not context:
             # Default context
             context = {"name": self.name, "id": 1, "model": self.model}
-        return [render_prompt(p, {"agent": context}) for p in self.system_prompts]
+
+        rendered_prompts: list[str] = []
+        for prompt in self.system_prompts:
+            match prompt:
+                case (str() as content) | StaticPromptConfig(content=content):
+                    rendered_prompts.append(render_prompt(content, {"agent": context}))
+                case FilePromptConfig(path=path, variables=variables):
+                    # Load and render Jinja template from file
+
+                    template_path = Path(path)
+                    if not template_path.is_absolute() and self.config_file_path:
+                        base_path = Path(self.config_file_path).parent
+                        template_path = base_path / path
+
+                    template_content = template_path.read_text()
+                    template_ctx = {"agent": context, **variables}
+                    rendered_prompts.append(render_prompt(template_content, template_ctx))
+                case LibraryPromptConfig(reference=reference):
+                    # This will be handled by the manifest's get_agent method
+                    # For now, just add a placeholder
+                    rendered_prompts.append(f"[LIBRARY:{reference}]")
+                case FunctionPromptConfig(function=function, arguments=arguments):
+                    # Import and call the function to get prompt content
+                    content = function(**arguments)
+                    rendered_prompts.append(render_prompt(content, {"agent": context}))
+
+        return rendered_prompts
 
     def get_config(self) -> Config:
         """Get configuration for this agent."""
@@ -340,8 +422,6 @@ class AgentConfig(NodeConfig):
     @staticmethod
     def _resolve_environment_path(env: str, config_file_path: str | None = None) -> str:
         """Resolve environment path from config store or relative path."""
-        from upath import UPath
-
         try:
             config_store = ConfigStore()
             return config_store.get_config(env)
