@@ -19,7 +19,6 @@ from llmling_agent_config.mcp_server import (
 )
 from llmling_agent_config.observability import ObservabilityConfig
 from llmling_agent_config.pool_server import MCPPoolServerConfig
-from llmling_agent_config.prompts import PromptConfig
 from llmling_agent_config.providers import BaseProviderConfig
 from llmling_agent_config.resources import (  # noqa: TC001
     ResourceConfig,
@@ -27,6 +26,7 @@ from llmling_agent_config.resources import (  # noqa: TC001
 )
 from llmling_agent_config.result_types import StructuredResponseConfig  # noqa: TC001
 from llmling_agent_config.storage import StorageConfig
+from llmling_agent_config.system_prompts import PromptLibraryConfig
 from llmling_agent_config.task import Job  # noqa: TC001
 from llmling_agent_config.teams import TeamConfig  # noqa: TC001
 from llmling_agent_config.ui import StdlibUIConfig, UIConfig  # noqa: TC001
@@ -109,7 +109,7 @@ class AgentsManifest(Schema):
     This MCP server configuration is used for the pool MCP server,
     which exposes pool functionality to other applications / clients."""
 
-    prompts: PromptConfig = Field(default_factory=PromptConfig)
+    prompts: PromptLibraryConfig = Field(default_factory=PromptLibraryConfig)
 
     model_config = ConfigDict(use_attribute_docstrings=True, extra="forbid")
 
@@ -236,13 +236,11 @@ class AgentsManifest(Schema):
             setattr(config, key, value)
 
         # Handle template rendering if context provided
-        if template_context:
-            # Apply name from context if not explicitly overridden
-            if "name" in template_context and "name" not in overrides:
-                config.name = template_context["name"]
+        if template_context and "name" in template_context and "name" not in overrides:
+            config.name = template_context["name"]
 
-            # Render system prompts
-            config.system_prompts = config.render_system_prompts(template_context)
+        # Note: system_prompts will be rendered during agent creation, not here
+        # config.system_prompts remains as PromptConfig objects
 
         self.agents[actual_name] = config
         return actual_name
@@ -296,17 +294,6 @@ class AgentsManifest(Schema):
         data["agents"] = resolved
         return data
 
-    @model_validator(mode="after")
-    def set_instrument_libraries(self) -> Self:
-        """Auto-set libraries to instrument based on used providers."""
-        if (
-            not self.observability.enabled
-            or self.observability.instrument_libraries is not None
-        ):
-            return self
-        self.observability.instrument_libraries = list(self.get_used_providers())
-        return self
-
     @property
     def node_names(self) -> list[str]:
         """Get list of all agent and team names."""
@@ -367,6 +354,7 @@ class AgentsManifest(Schema):
     def get_agent[TAgentDeps](
         self, name: str, deps: TAgentDeps | None = None
     ) -> AnyAgent[TAgentDeps, Any]:
+        # TODO: Make this method async to support async function prompts
         from llmling import RuntimeConfig
 
         from llmling_agent import Agent, AgentContext
@@ -388,17 +376,49 @@ class AgentsManifest(Schema):
             # confirmation_callback=confirmation_callback,
         )
 
-        sys_prompts = config.system_prompts.copy()
-        # Library prompts
-        if config.library_system_prompts:
-            for prompt_ref in config.library_system_prompts:
-                try:
-                    content = self.prompt_manager.get_sync(prompt_ref)
+        # Resolve system prompts with new PromptConfig types
+        from pathlib import Path
+
+        from llmling_agent_config.system_prompts import (
+            FilePromptConfig,
+            FunctionPromptConfig,
+            LibraryPromptConfig,
+            StaticPromptConfig,
+        )
+
+        sys_prompts: list[str] = []
+        for prompt in config.system_prompts:
+            match prompt:
+                case (str() as sys_prompt) | StaticPromptConfig(content=sys_prompt):
+                    sys_prompts.append(sys_prompt)
+                case FilePromptConfig(path=path, variables=variables):
+                    template_path = Path(path)  # Load template from file
+                    if not template_path.is_absolute() and config.config_file_path:
+                        template_path = Path(config.config_file_path).parent / path
+
+                    template_content = template_path.read_text()
+                    if variables:  # Apply variables if any
+                        from jinja2 import Template
+
+                        template = Template(template_content)
+                        content = template.render(**variables)
+                    else:
+                        content = template_content
+                    sys_prompts.append(content)
+                case LibraryPromptConfig(reference=reference):
+                    try:  # Load from library
+                        content = self.prompt_manager.get_sync(reference)
+                        sys_prompts.append(content)
+                    except Exception as e:
+                        msg = (
+                            f"Failed to load library prompt {reference!r} "
+                            f"for agent {name}"
+                        )
+                        logger.exception(msg)
+                        raise ValueError(msg) from e
+                case FunctionPromptConfig(function=function, arguments=arguments):
+                    content = function(**arguments)  # Call function to get prompt content
                     sys_prompts.append(content)
-                except Exception as e:
-                    msg = f"Failed to load library prompt {prompt_ref!r} for agent {name}"
-                    logger.exception(msg)
-                    raise ValueError(msg) from e
         # Create agent with runtime and context
         agent = Agent[Any](
             runtime=runtime,
@@ -425,10 +445,8 @@ class AgentsManifest(Schema):
 
         for agent_config in self.agents.values():
             match agent_config.provider:
-                case "pydantic_ai":
-                    providers.add("pydantic_ai")
-                case BaseProviderConfig():
-                    providers.add(agent_config.provider.type)
+                case ("pydantic_ai" as typ) | BaseProviderConfig(type=typ):
+                    providers.add(typ)
         return providers
 
     @classmethod
