@@ -7,7 +7,7 @@ from typing import TYPE_CHECKING, Any
 
 from llmling_agent.log import get_logger
 from llmling_agent.prompts.builtin_provider import BuiltinPromptProvider
-from llmling_agent.utils.tasks import TaskManagerMixin
+from llmling_agent.utils.tasks import TaskManager
 from llmling_agent_config.prompt_hubs import (
     LangfuseConfig,
     OpenLITConfig,
@@ -18,7 +18,7 @@ from llmling_agent_config.prompt_hubs import (
 
 if TYPE_CHECKING:
     from llmling_agent.prompts.base import BasePromptProvider
-    from llmling_agent_config.prompts import PromptConfig
+    from llmling_agent_config.system_prompts import PromptLibraryConfig
 
 logger = get_logger(__name__)
 
@@ -35,7 +35,7 @@ def parse_prompt_reference(reference: str) -> tuple[str, str, str | None, dict[s
     """
     provider = "builtin"
     version = None
-    variables = {}
+    variables: dict[str, str] = {}
 
     # Split provider and rest
     if ":" in reference:
@@ -59,7 +59,7 @@ def parse_prompt_reference(reference: str) -> tuple[str, str, str | None, dict[s
     return provider, identifier.strip(), version, variables
 
 
-class PromptManager(TaskManagerMixin):
+class PromptManager:
     """Manages multiple prompt providers.
 
     Handles:
@@ -68,7 +68,7 @@ class PromptManager(TaskManagerMixin):
     - Access to prompts from different sources
     """
 
-    def __init__(self, config: PromptConfig):
+    def __init__(self, config: PromptLibraryConfig):
         """Initialize prompt manager.
 
         Args:
@@ -76,6 +76,7 @@ class PromptManager(TaskManagerMixin):
         """
         super().__init__()
         self.config = config
+        self.task_manager = TaskManager()
         self.providers: dict[str, BasePromptProvider] = {}
 
         # Always register builtin provider
@@ -89,7 +90,7 @@ class PromptManager(TaskManagerMixin):
 
                     self.providers["langfuse"] = LangfusePromptHub(provider_config)
                 case OpenLITConfig():
-                    from llmling_agent_prompts.openlit import OpenLITProvider
+                    from llmling_agent_prompts.openlit_provider import OpenLITProvider
 
                     self.providers["openlit"] = OpenLITProvider(provider_config)
                 case PromptLayerConfig():
@@ -184,17 +185,19 @@ class PromptManager(TaskManagerMixin):
         Returns:
             Dict mapping provider names to their available prompts
         """
-        result = {}
         providers = {provider: self.providers[provider]} if provider else self.providers
 
-        for name, p in providers.items():
-            try:
-                prompts = await p.list_prompts()
-                result[name] = prompts
-            except Exception:
+        if not providers:
+            return {}
+        # Get prompts from providers concurrently
+        result = {}
+        coros = [p.list_prompts() for p in providers.values()]
+        gathered_results = await asyncio.gather(*coros, return_exceptions=True)
+        for (name, _), prompts in zip(providers.items(), gathered_results, strict=False):
+            if isinstance(prompts, BaseException):
                 logger.exception("Failed to list prompts from %s", name)
                 continue
-
+            result[name] = prompts
         return result
 
     def cleanup(self):
