@@ -6,7 +6,7 @@ import asyncio
 from typing import TYPE_CHECKING, Self
 
 from llmling_agent.log import get_logger
-from llmling_agent.utils.tasks import TaskManagerMixin
+from llmling_agent.utils.tasks import TaskManager
 from llmling_agent_config.storage import (
     FileStorageConfig,
     Mem0Config,
@@ -33,7 +33,7 @@ if TYPE_CHECKING:
 logger = get_logger(__name__)
 
 
-class StorageManager(TaskManagerMixin):
+class StorageManager:
     """Manages multiple storage providers.
 
     Handles:
@@ -50,6 +50,7 @@ class StorageManager(TaskManagerMixin):
             config: Storage configuration including providers and filters
         """
         self.config = config
+        self.task_manager = TaskManager()
         self.providers = [
             self._create_provider(cfg) for cfg in self.config.effective_providers
         ]
@@ -75,7 +76,7 @@ class StorageManager(TaskManagerMixin):
                 errors.append(e)
                 logger.exception("Error cleaning up provider: %r", provider)
 
-        await self.cleanup_tasks()
+        await self.task_manager.cleanup_tasks()
 
         if errors:
             msg = "Provider cleanup errors"
@@ -118,14 +119,12 @@ class StorageManager(TaskManagerMixin):
         )
 
         match provider_config:
-            case SQLStorageConfig():
+            case SQLStorageConfig(url=url, pool_size=pool_size):
                 from sqlmodel import create_engine
 
                 from llmling_agent_storage.sql_provider import SQLModelProvider
 
-                engine = create_engine(
-                    provider_config.url, pool_size=provider_config.pool_size
-                )
+                engine = create_engine(url, pool_size=pool_size)
                 return SQLModelProvider(provider_config, engine)
             case FileStorageConfig():
                 from llmling_agent_storage.file_provider import FileProvider
@@ -226,7 +225,7 @@ class StorageManager(TaskManagerMixin):
 
         for provider in self.providers:
             if provider.should_log_agent(name or "no name"):
-                self.create_task(
+                self.task_manager.create_task(
                     provider.log_message(
                         conversation_id=conversation_id,
                         message_id=message_id,
@@ -252,7 +251,7 @@ class StorageManager(TaskManagerMixin):
             return
 
         for provider in self.providers:
-            self.create_task(
+            self.task_manager.create_task(
                 provider.log_conversation(
                     conversation_id=conversation_id,
                     node_name=node_name,
@@ -272,7 +271,7 @@ class StorageManager(TaskManagerMixin):
             return
 
         for provider in self.providers:
-            self.create_task(
+            self.task_manager.create_task(
                 provider.log_tool_call(
                     conversation_id=conversation_id,
                     message_id=message_id,
@@ -294,7 +293,7 @@ class StorageManager(TaskManagerMixin):
             return
 
         for provider in self.providers:
-            self.create_task(
+            self.task_manager.create_task(
                 provider.log_command(
                     agent_name=agent_name,
                     session_id=session_id,
@@ -315,7 +314,7 @@ class StorageManager(TaskManagerMixin):
     ):
         """Log context message to all providers."""
         for provider in self.providers:
-            self.create_task(
+            self.task_manager.create_task(
                 provider.log_context_message(
                     conversation_id=conversation_id,
                     content=content,
@@ -380,11 +379,13 @@ class StorageManager(TaskManagerMixin):
     # Sync wrappers
     def reset_sync(self, *args, **kwargs) -> tuple[int, int]:
         """Sync wrapper for reset."""
-        return self.run_task_sync(self.reset(*args, **kwargs))
+        return self.task_manager.run_task_sync(self.reset(*args, **kwargs))
 
     def get_conversation_counts_sync(self, *args, **kwargs) -> tuple[int, int]:
         """Sync wrapper for get_conversation_counts."""
-        return self.run_task_sync(self.get_conversation_counts(*args, **kwargs))
+        return self.task_manager.run_task_sync(
+            self.get_conversation_counts(*args, **kwargs)
+        )
 
     def log_conversation_sync(self, *args, **kwargs):
         """Sync wrapper for log_conversation."""
