@@ -10,7 +10,7 @@ from llmling_agent_acp.command_bridge import ACPCommandBridge
 from llmling_agent_acp.converters import (
     convert_acp_mcp_server_to_config,
     from_content_blocks,
-    to_session_updates,
+    to_agent_text_notification,
 )
 
 
@@ -22,5 +22,5 @@ __all__ = [
     "LLMlingACPAgent",
     "convert_acp_mcp_server_to_config",
     "from_content_blocks",
-    "to_session_updates",
+    "to_agent_text_notification",
 ]
