@@ -23,9 +23,10 @@ from acp.schema import (
     SetSessionModeResponse,
 )
 from llmling_agent.log import get_logger
-from llmling_agent.utils.tasks import TaskManagerMixin
+from llmling_agent.utils.tasks import TaskManager
+from llmling_agent_acp.acp_commands import get_acp_commands
 from llmling_agent_acp.command_bridge import ACPCommandBridge
-from llmling_agent_acp.converters import to_session_updates
+from llmling_agent_acp.converters import to_agent_text_notification
 from llmling_agent_acp.session_manager import ACPSessionManager
 from llmling_agent_commands import get_commands
 
@@ -72,6 +73,7 @@ class LLMlingACPAgent(ACPAgent):
         file_access: bool = True,
         terminal_access: bool = True,
         usage_limits: UsageLimits | None = None,
+        debug_commands: bool = False,
     ) -> None:
         """Initialize ACP agent implementation.
 
@@ -83,6 +85,7 @@ class LLMlingACPAgent(ACPAgent):
             file_access: Whether agent can access filesystem
             terminal_access: Whether agent can use terminal
             usage_limits: Optional usage limits for model requests and tokens
+            debug_commands: Whether to enable debug slash commands for testing
         """
         self.connection = connection
         self.agent_pool = agent_pool
@@ -92,20 +95,28 @@ class LLMlingACPAgent(ACPAgent):
         self.terminal_access = terminal_access
         self.client: Client = connection
         self.usage_limits = usage_limits
+        self.debug_commands = debug_commands
         self.client_capabilities: ClientCapabilities | None = None
         command_store = CommandStore(enable_system_commands=True)
-        command_store._initialize_sync()  # Ensure store is initialized
+        command_store._initialize_sync()
 
-        for command in get_commands():
+        commands_to_register = [*get_commands(), *get_acp_commands()]
+        if debug_commands:
+            from llmling_agent_acp.debug_commands import get_debug_commands
+
+            commands_to_register.extend(get_debug_commands())
+
+        for command in commands_to_register:
             command_store.register_command(command)
-
         self.command_bridge = ACPCommandBridge(command_store)
         self.session_manager = ACPSessionManager(command_bridge=self.command_bridge)
-        self.tasks = TaskManagerMixin()
+        self.tasks = TaskManager()
 
         self._initialized = False
         agent_count = len(self.agent_pool.agents)
         logger.info("Created ACP agent implementation with %d agents", agent_count)
+        if debug_commands:
+            logger.info("Debug slash commands enabled for ACP testing")
 
         # Note: Tool registration happens after initialize() when we know client caps
 
@@ -161,14 +172,14 @@ class LLMlingACPAgent(ACPAgent):
             # Create session modes from available agents
             modes = [
                 SessionMode(
-                    id=agent_name,
-                    name=self.agent_pool.get_agent(agent_name).name,
+                    id=name,
+                    name=name,
                     description=(
-                        self.agent_pool.get_agent(agent_name).description
-                        or f"Switch to {self.agent_pool.get_agent(agent_name).name} agent"
+                        self.agent_pool.get_agent(name).description
+                        or f"Switch to {name} agent"
                     ),
                 )
-                for agent_name in agent_names
+                for name in agent_names
             ]
 
             state = SessionModeState(current_mode_id=default_name, available_modes=modes)
@@ -251,7 +262,7 @@ class LLMlingACPAgent(ACPAgent):
         except Exception as e:
             logger.exception("Failed to process prompt for session %s", params.session_id)
             msg = f"Error processing prompt: {e}"
-            for update in to_session_updates(msg, params.session_id):
+            if update := to_agent_text_notification(msg, params.session_id):
                 try:
                     await self.connection.session_update(update)
                 except Exception:
