@@ -2,19 +2,22 @@
 
 from __future__ import annotations
 
-from contextlib import asynccontextmanager
 from dataclasses import asdict
 from decimal import Decimal
 from functools import wraps
-from typing import TYPE_CHECKING, Any, cast, get_args, get_origin
+from typing import TYPE_CHECKING, Any, get_args, get_origin
 
 from llmling_models import AllModels, infer_model
+import logfire
 from pydantic_ai import Agent as PydanticAgent
 import pydantic_ai._function_schema
-from pydantic_ai.messages import ModelResponse
-from pydantic_ai.models import KnownModelName, Model
-from pydantic_ai.result import StreamedRunResult
-from pydantic_ai.run import AgentRun
+from pydantic_ai.messages import (
+    FunctionToolCallEvent,
+    PartStartEvent,
+    TextPartDelta,
+    ToolCallPart,
+)
+from pydantic_ai.models import KnownModelName
 from pydantic_ai.tools import GenerateToolJsonSchema, RunContext
 from pydantic_ai.usage import UsageLimits as PydanticAiUsageLimits
 
@@ -22,26 +25,26 @@ from llmling_agent.agent.context import AgentContext
 from llmling_agent.common_types import ModelProtocol
 from llmling_agent.log import get_logger
 from llmling_agent.messaging.messages import TokenCost, TokenUsage
-from llmling_agent.observability import track_action
 from llmling_agent.tasks.exceptions import (
     ChainAbortedError,
     RunAbortedError,
     ToolSkippedError,
 )
 from llmling_agent.utils.inspection import execute, has_argument_type
-from llmling_agent_providers.base import AgentLLMProvider, ProviderResponse
+from llmling_agent_providers.base import AgentProvider, ProviderResponse
 from llmling_agent_providers.pydanticai.utils import (
     convert_prompts_to_user_content,
-    format_part,
     get_tool_calls,
-    to_model_message,
+    to_model_request,
 )
 
 
 if TYPE_CHECKING:
     from collections.abc import AsyncIterator, Awaitable, Callable
 
+    from pydantic_ai import AgentStreamEvent
     from pydantic_ai.agent import AgentRunResult
+    from pydantic_ai.run import AgentRunResultEvent
 
     from llmling_agent.common_types import EndStrategy, ModelType
     from llmling_agent.messaging.messages import ChatMessage
@@ -81,7 +84,7 @@ def _is_call_ctx(annotation: Any) -> bool:
 pydantic_ai._function_schema._is_call_ctx = _is_call_ctx  # type: ignore
 
 
-class PydanticAIProvider[TDeps](AgentLLMProvider[TDeps]):
+class PydanticAIProvider(AgentProvider[Any]):
     """Provider using pydantic-ai as backend."""
 
     NAME = "pydantic_ai"
@@ -113,11 +116,12 @@ class PydanticAIProvider[TDeps](AgentLLMProvider[TDeps]):
             model_settings: Additional model-specific settings
         """
         super().__init__(
-            model=model,
-            model_settings=model_settings,
-            debug=debug,
+            name=name,
             context=context,
+            debug=debug,
         )
+        self.model_settings = model_settings or {}
+        self._model = model
         self._kwargs: dict[str, Any] = dict(
             model=model,
             name=name,
@@ -128,6 +132,9 @@ class PydanticAIProvider[TDeps](AgentLLMProvider[TDeps]):
             defer_model_check=defer_model_check,
         )
 
+    def __repr__(self) -> str:
+        return f"{self.__class__.__name__}(model={self.model_name})"
+
     async def get_model_names(self) -> list[str]:
         """Get list of all known model names."""
         return list(get_args(KnownModelName)) + list(get_args(AllModels))
@@ -151,15 +158,15 @@ class PydanticAIProvider[TDeps](AgentLLMProvider[TDeps]):
                 agent.tool(wrapped)
             elif has_argument_type(wrapped, AgentContext):
                 agent._function_toolset.add_function(
-                    wrapped,
-                    True,
-                    None,
-                    1,
-                    None,
-                    "auto",
-                    False,
-                    GenerateToolJsonSchema,
-                    None,
+                    func=wrapped,
+                    takes_ctx=True,
+                    name=None,
+                    retries=1,
+                    prepare=None,
+                    docstring_format="auto",
+                    require_parameter_descriptions=False,
+                    schema_generator=GenerateToolJsonSchema,
+                    strict=None,
                 )
             else:
                 agent.tool_plain(wrapped)
@@ -243,7 +250,7 @@ class PydanticAIProvider[TDeps](AgentLLMProvider[TDeps]):
         wrapped.__name__ = tool.name
         return wrapped
 
-    @track_action("Pydantic-AI call. model: {model} result type {result_type}.")
+    @logfire.instrument("Pydantic-AI call. model: {model} result type {result_type}.")
     async def generate_response(
         self,
         *prompts: str | Content,
@@ -262,48 +269,64 @@ class PydanticAIProvider[TDeps](AgentLLMProvider[TDeps]):
         if isinstance(use_model, str):
             use_model = infer_model(use_model)
             self.model_changed.emit(use_model)
+
+        # Create event stream handler for real-time tool signals
+        tool_dict = {i.name: i for i in tools or []}
+
+        async def event_stream_handler(ctx, events):
+            async for event in events:
+                if isinstance(event, FunctionToolCallEvent):
+                    # Extract tool call info and emit signal immediately
+                    call = event.part
+                    if call.tool_name in tool_dict:
+                        tool_call_info = self._create_tool_call_info(
+                            call, tool_dict, message_id
+                        )
+                        tool_call_info.message_id = message_id
+                        tool_call_info.context_data = (
+                            self._context.data if self._context else None
+                        )
+
+                        self.tool_used.emit(tool_call_info)
+
         try:
             # Convert prompts to pydantic-ai format
             converted_prompts = await convert_prompts_to_user_content(prompts)
 
-            # Run with complete history
+            # Run with complete history and event handler
             to_use = model or self.model
             to_use = infer_model(to_use) if isinstance(to_use, str) else to_use
             limits = asdict(usage_limits) if usage_limits else {}
             result: AgentRunResult = await agent.run(
                 converted_prompts,  # Pass converted prompts
                 deps=self._context,  # type: ignore
-                message_history=[to_model_message(m) for m in message_history],
+                message_history=[to_model_request(m) for m in message_history],
                 model=to_use,  # type: ignore
                 output_type=result_type or str,
                 model_settings=self.model_settings,  # type: ignore
                 usage_limits=PydanticAiUsageLimits(**limits),
+                event_stream_handler=event_stream_handler,
             )
 
-            # Extract tool calls and set message_id
+            # Extract tool calls for final response
+            # (signals already emitted via event handler)
             new_msgs = result.new_messages()
-            tool_dict = {i.name: i for i in tools or []}
             tool_calls = get_tool_calls(new_msgs, tool_dict, agent_name=self.name)
             for call in tool_calls:
                 call.message_id = message_id
                 call.context_data = self._context.data if self._context else None
-                self.tool_used.emit(call)
 
-            resolved_model = (
-                use_model.model_name if isinstance(use_model, Model) else str(use_model)
-            )
+            # Get the actual model name from pydantic-ai response
+            resolved_model = result.response.model_name or ""
             usage = result.usage()
-            responses = [m for m in new_msgs if isinstance(m, ModelResponse)]
-            try:
-                cost_sum = sum(i.cost().total_price for i in responses)
-            except LookupError:
-                cost_sum = Decimal(0)
+            cost = await TokenCost.from_usage(model=resolved_model, usage=usage)
+            total = cost.total_cost if cost else Decimal(0)
             token_usage = TokenUsage(
                 total=usage.total_tokens,
                 prompt=usage.input_tokens,
                 completion=usage.output_tokens,
             )
-            cost_info = TokenCost(token_usage=token_usage, total_cost=Decimal(cost_sum))
+            cost_info = TokenCost(token_usage=token_usage, total_cost=total)
             return ProviderResponse(
                 content=result.output,
                 tool_calls=tool_calls,
@@ -357,70 +380,23 @@ class PydanticAIProvider[TDeps](AgentLLMProvider[TDeps]):
                 msg = f"Invalid model type: {model}"
                 raise ValueError(msg)
 
-    @asynccontextmanager
-    async def iterate_run[TResult](
-        self,
-        *prompts: str | Content,  # Provider signature
-        message_id: str,
-        message_history: list[ChatMessage[Any]],  # Use Any generic
-        tools: list[Tool] | None = None,
-        result_type: type[TResult] | None = None,
-        usage_limits: UsageLimits | None = None,
-        model: ModelType = None,
-        system_prompt: str | None = None,
-        **kwargs: Any,  # Analogous kwargs
-    ) -> AsyncIterator[AgentRun[TDeps, TResult]]:  # Use Any generic for now
-        """Starts an iterable agent run using pydantic-ai's Agent.iter."""
-        agent = await self.get_agent(system_prompt or "", tools=tools or [])
-        use_model = model or self.model
-        original_model_signal = None
-        if model:
-            original_model_signal = self.model
-            if isinstance(use_model, str):
-                use_model = infer_model(use_model)
-            # Don't emit model_changed here, agent.iter takes model directly
-            # self.model_changed.emit(use_model) # NO EMIT NEEDED HERE
-        elif isinstance(use_model, str):
-            use_model = infer_model(use_model)
+    def _create_tool_call_info(
+        self, tool_part: ToolCallPart, tool_dict: dict, message_id: str
+    ):
+        """Create ToolCallInfo from tool call part."""
+        from llmling_agent.tools.tool_call_info import ToolCallInfo
 
-        limits = asdict(usage_limits) if usage_limits else {}
-        pydantic_limits = PydanticAiUsageLimits(**limits)
-        resolved_model_name = (
-            use_model.model_name if isinstance(use_model, Model) else str(use_model)
+        return ToolCallInfo(
+            tool_name=tool_part.tool_name,
+            args=tool_part.args_as_dict(),
+            result=None,  # Will be filled when tool execution completes
+            agent_name=self.name,
+            tool_call_id=tool_part.tool_call_id,
+            message_id=message_id,
+            context_data=self._context.data if self._context else None,
         )
 
-        try:
-            converted_prompts = await convert_prompts_to_user_content(prompts)
-            model_messages = [to_model_message(m) for m in message_history]
-            msg = "Starting PydanticAI agent iteration run_id=%s with model=%s"
-            logger.debug(msg, message_id, resolved_model_name)
-            async with agent.iter(
-                converted_prompts,
-                deps=self._context,
-                message_history=model_messages,
-                model=model or self.model,  # type: ignore
-                output_type=result_type or str,
-                model_settings=self.model_settings,  # type: ignore
-                usage_limits=pydantic_limits,
-                **kwargs,
-            ) as agent_run:
-                # 7. Post-processing (Minimal for iter, just yield)
-                # Add model name to the run object if possible/needed?
-                # setattr(agent_run, 'model_name', resolved_model_name)
-
-                yield cast(AgentRun[Any, Any], agent_run)  # Yield the run object
-        except Exception as e:
-            # Catch potential errors from argument mismatches in agent.iter
-            logger.exception("Error calling agent.iter")
-            msg = f"Iteration setup error: {e}"
-            raise RuntimeError(msg) from e
-        finally:
-            if original_model_signal and isinstance(original_model_signal, str):
-                original_model_signal = infer_model(original_model_signal)
-            # self.model_changed.emit(original_model_signal)
-
-    @asynccontextmanager
-    async def stream_response(  # type: ignore[override]
+    async def stream_events(
         self,
         *prompts: str | Content,
         message_history: list[ChatMessage],
@@ -431,8 +407,8 @@ class PydanticAIProvider[TDeps](AgentLLMProvider[TDeps]):
         system_prompt: str | None = None,
         usage_limits: UsageLimits | None = None,
         **kwargs: Any,
-    ) -> AsyncIterator[StreamedRunResult]:  # type: ignore[type-var]
-        """Stream response using pydantic-ai."""
+    ) -> AsyncIterator[AgentStreamEvent | AgentRunResultEvent]:
+        """Stream events directly from pydantic-ai without wrapper complexity."""
         agent = await self.get_agent(system_prompt or "", tools=tools or [])
         use_model = model or self.model
         limits = asdict(usage_limits) if usage_limits else {}
@@ -444,11 +420,13 @@ class PydanticAIProvider[TDeps](AgentLLMProvider[TDeps]):
 
         # Convert prompts to pydantic-ai format
         converted_prompts = await convert_prompts_to_user_content(prompts)
-
         # Convert all messages to pydantic-ai format
-        model_messages = [to_model_message(m) for m in message_history]
+        model_messages = [to_model_request(m) for m in message_history]
 
-        async with agent.run_stream(
+        tool_dict = {i.name: i for i in tools or []}
+
+        # Stream events directly from pydantic-ai
+        async for event in agent.run_stream_events(
             converted_prompts,
             deps=self._context,
             message_history=model_messages,
@@ -456,71 +434,101 @@ class PydanticAIProvider[TDeps](AgentLLMProvider[TDeps]):
             output_type=result_type or str,
             model_settings=self.model_settings,  # type: ignore
             usage_limits=PydanticAiUsageLimits(**limits),
-        ) as stream_result:
-            stream_result = cast(StreamedRunResult[AgentContext[Any], Any], stream_result)
-            original_stream = stream_result.stream_output
-            original_text_stream = stream_result.stream_text
-            resolved_model = (
-                use_model.model_name if isinstance(use_model, Model) else str(use_model)
-            )
-            stream_result.model_name = resolved_model  # type: ignore
+        ):
+            # Emit signals for external consumers
+            match event:
+                case (
+                    PartStartEvent(part=ToolCallPart() as tool_part)
+                    | FunctionToolCallEvent(part=tool_part)
+                ):
+                    call_info = self._create_tool_call_info(
+                        tool_part, tool_dict, message_id
+                    )
+                    self.tool_used.emit(call_info)
+            yield event
 
-            def get_wrapped_stream(fn):
-                async def wrapped_stream(*args, **kwargs):
-                    last_content = None
-                    async for chunk in fn(*args, **kwargs):
-                        # Only emit if content has changed
-                        if chunk != last_content:
-                            self.chunk_streamed.emit(str(chunk), message_id)
-                            last_content = chunk
-                            yield chunk
-
-                    if stream_result.is_complete:
-                        self.chunk_streamed.emit("", message_id)
-                        messages = stream_result.new_messages()
-                        tool_dict = {i.name: i for i in tools or []}
-                        # Extract and update tool calls
-                        tool_calls = get_tool_calls(
-                            messages, tool_dict, agent_name=self.name
-                        )
-                        for call in tool_calls:
-                            call.message_id = message_id
-                            call.context_data = (
-                                self._context.data if self._context else None
-                            )
-                            self.tool_used.emit(call)
-                        # Format final content
-                        responses = [m for m in messages if isinstance(m, ModelResponse)]
-                        parts = [p for msg in responses for p in msg.parts]
-                        content = "\n".join(format_part(p) for p in parts)
-                        # Update stream result with formatted content
-                        stream_result.formatted_content = content  # type: ignore
-
-                return wrapped_stream
-
-            if model:
-                original = self.model
-                if isinstance(original, str):
-                    original = infer_model(original)
-                self.model_changed.emit(original)
-
-            stream_result.stream_output = get_wrapped_stream(original_stream)  # type: ignore
-            stream_result.stream_text = get_wrapped_stream(original_text_stream)  # type: ignore
-            yield stream_result  # type: ignore
+        # Reset model signal if needed
+        if model:
+            original = self.model
+            if isinstance(original, str):
+                original = infer_model(original)
+            self.model_changed.emit(original)
 
 
 if __name__ == "__main__":
     import asyncio
 
+    from llmling_agent.tools import Tool
+
     async def main():
-        provider = PydanticAIProvider[None](model="openai:gpt-5-nano")
+        provider = PydanticAIProvider(model="openai:gpt-5-nano")
 
-        # Example usage
-        response = await provider.generate_response(
-            "Hello, how are you?",
-            message_id="test",
+        def write_poem(text: str) -> bool:
+            """The ultimate poem writing tool."""
+            return True
+
+        tool = Tool.from_callable(write_poem)
+        print("Testing our provider's stream_events method...")
+        chunks = []
+        final_result = None
+        async for event in provider.stream_events(
+            "Write a long poem using the tool",
             message_history=[],
-        )
-        print(response.cost_and_usage)
+            message_id="test-123",
+            tools=[tool],
+        ):
+            from pydantic_ai import AgentRunResultEvent, PartDeltaEvent, ToolCallPartDelta
 
-    asyncio.run(main())
+            match event:
+                case PartDeltaEvent(delta=TextPartDelta(content_delta=delta)):
+                    chunks.append(delta)
+                    print(f"Text chunk: '{delta}'")
+                case PartDeltaEvent(delta=ToolCallPartDelta(content_delta=delta)):
+                    chunks.append(delta)
+                    print(f"Tool call chunk: '{delta}'")
+                case AgentRunResultEvent(result=result):
+                    final_result = result
+                    print(f"Final result: {result}")
+
+        print(f"\nTotal chunks collected: {len(chunks)}")
+        print(f"Full content: {''.join(chunks)}")
+        assert final_result is not None
+        print(f"Final result available: {final_result is not None}")
+        print(f"Model: {final_result.response.model_name}")
+
+    # Test real-time tool signal emission
+    async def test_tool_signals():
+        print("\nTesting real-time tool signal emission...")
+
+        # Create provider with signal handler
+        signals_received = []
+
+        def on_tool_used(tool_info):
+            signals_received.append(tool_info)
+            print(f"🔧 Signal received: {tool_info.tool_name}({tool_info.args})")
+
+        provider = PydanticAIProvider()
+        provider.tool_used.connect(on_tool_used)
+
+        # Create a simple tool
+        def get_weather(city: str) -> str:
+            return f"It's sunny in {city}"
+
+        tools = [Tool.from_callable(get_weather)]
+
+        try:
+            response = await provider.generate_response(
+                "What's the weather like in Paris?",
+                message_id="test-123",
+                model="openai:gpt-4o-mini",
+                message_history=[],
+                tools=tools,
+            )
+            print(f"Response: {response.content}")
+            print(f"Signals received: {len(signals_received)}")
+
+        except Exception as e:  # noqa: BLE001
+            print(f"Error: {e}")
+
+    # asyncio.run(main())
+    asyncio.run(test_tool_signals())
