@@ -2,7 +2,7 @@
 
 from __future__ import annotations
 
-from typing import TYPE_CHECKING, Any
+from typing import TYPE_CHECKING, Any, overload
 from uuid import uuid4
 
 from pydantic_ai import messages as _messages
@@ -25,6 +25,12 @@ from llmling_agent.tools import ToolCallInfo
 if TYPE_CHECKING:
     from collections.abc import Sequence
 
+    from pydantic_ai.mcp import (
+        MCPServer,
+        MCPServerSSE,
+        MCPServerStdio,
+        MCPServerStreamableHTTP,
+    )
     from pydantic_ai.messages import (
         ModelMessage,
         ModelRequestPart,
@@ -35,6 +41,12 @@ if TYPE_CHECKING:
     from llmling_agent.common_types import MessageRole
     from llmling_agent.models.content import Content
     from llmling_agent.tools.base import Tool
+    from llmling_agent_config.mcp_server import (
+        MCPServerConfig,
+        SSEMCPServerConfig,
+        StdioMCPServerConfig,
+        StreamableHTTPMCPServerConfig,
+    )
 
 
 def format_part(  # noqa: PLR0911
@@ -210,7 +222,7 @@ def convert_model_message(  # noqa: PLR0911
             raise ValueError(msg)
 
 
-def to_model_message(message: ChatMessage[str | Content]) -> ModelMessage:
+def to_model_request(message: ChatMessage[str | Content]) -> ModelRequest:
     """Convert ChatMessage to pydantic-ai ModelMessage."""
     import anyenv
 
@@ -260,3 +272,66 @@ async def convert_prompts_to_user_content(
             result.append(p_content)
 
     return result
+
+
+@overload
+def mcp_config_to_pydantic_ai(config: StdioMCPServerConfig) -> MCPServerStdio: ...
+
+
+@overload
+def mcp_config_to_pydantic_ai(config: SSEMCPServerConfig) -> MCPServerSSE: ...
+
+
+@overload
+def mcp_config_to_pydantic_ai(
+    config: StreamableHTTPMCPServerConfig,
+) -> MCPServerStreamableHTTP: ...
+
+
+@overload
+def mcp_config_to_pydantic_ai(config: MCPServerConfig) -> MCPServer: ...
+
+
+def mcp_config_to_pydantic_ai(config: MCPServerConfig) -> MCPServer:
+    """Convert llmling-agent MCP server config to pydantic-ai MCP server.
+
+    Args:
+        config: The MCP server configuration to convert
+
+    Returns:
+        A pydantic-ai MCP server instance
+
+    Raises:
+        ValueError: If server type is not supported
+    """
+    from pydantic_ai.mcp import MCPServerSSE, MCPServerStdio, MCPServerStreamableHTTP
+
+    match config.type:
+        case "stdio":
+            return MCPServerStdio(
+                command=config.command,
+                args=config.args,
+                env=config.get_env_vars() if config.env else None,
+                id=config.name,
+                timeout=config.timeout,
+            )
+
+        case "sse":
+            return MCPServerSSE(
+                url=config.url,
+                headers=config.headers,
+                id=config.name,
+                timeout=config.timeout,
+            )
+
+        case "streamable-http":
+            return MCPServerStreamableHTTP(
+                url=config.url,
+                headers=config.headers,
+                id=config.name,
+                timeout=config.timeout,
+            )
+
+        case _:
+            msg = f"Unsupported MCP server type: {config.type}"
+            raise ValueError(msg)
