@@ -3,7 +3,6 @@
 from __future__ import annotations
 
 from datetime import UTC, datetime
-import importlib.util
 from typing import Any
 from uuid import uuid4
 
@@ -15,13 +14,10 @@ from sqlalchemy.types import TypeDecorator
 from sqlmodel import JSON, Field, SQLModel
 from sqlmodel.main import SQLModelConfig
 
-from llmling_agent.common_types import JsonValue  # noqa: TC001
+from llmling_agent.common_types import JsonValue
 from llmling_agent.utils.now import get_now
 
 
-REFLEX_INSTALLED = importlib.util.find_spec("reflex") is not None
-
-
 class UTCDateTime(TypeDecorator):
     """Stores DateTime as UTC."""
 
@@ -70,8 +66,7 @@ class CommandHistory(AsyncAttrs, SQLModel, table=True):  # type: ignore[call-arg
     )
     """When the command was executed"""
 
-    if not REFLEX_INSTALLED:
-        model_config = SQLModelConfig(use_attribute_docstrings=True)  # pyright: ignore[reportCallIssue]
+    model_config = SQLModelConfig(use_attribute_docstrings=True)  # pyright: ignore[reportCallIssue]
 
 
 class MessageLog(Schema):
@@ -169,8 +164,7 @@ class Message(AsyncAttrs, SQLModel, table=True):  # type: ignore[call-arg]
     checkpoint_data: dict[str, Any] | None = Field(default=None, sa_column=Column(JSON))
     """A dictionary of checkpoints (name -> metadata)."""
 
-    if not REFLEX_INSTALLED:
-        model_config = SQLModelConfig(use_attribute_docstrings=True)  # pyright: ignore[reportCallIssue]
+    model_config = SQLModelConfig(use_attribute_docstrings=True)  # pyright: ignore[reportCallIssue]
 
 
 class ToolCall(AsyncAttrs, SQLModel, table=True):  # type: ignore[call-arg]
@@ -202,8 +196,7 @@ class ToolCall(AsyncAttrs, SQLModel, table=True):  # type: ignore[call-arg]
     result: str = Field(...)
     """Result returned by the tool"""
 
-    if not REFLEX_INSTALLED:
-        model_config = SQLModelConfig(use_attribute_docstrings=True)  # pyright: ignore[reportCallIssue]
+    model_config = SQLModelConfig(use_attribute_docstrings=True)  # pyright: ignore[reportCallIssue]
 
 
 class Conversation(AsyncAttrs, SQLModel, table=True):  # type: ignore[call-arg]
@@ -226,5 +219,4 @@ class Conversation(AsyncAttrs, SQLModel, table=True):  # type: ignore[call-arg]
     total_cost: float = 0.0
     """Total cost of this conversation in USD"""
 
-    if not REFLEX_INSTALLED:
-        model_config = SQLModelConfig(use_attribute_docstrings=True)  # pyright: ignore[reportCallIssue]
+    model_config = SQLModelConfig(use_attribute_docstrings=True)  # pyright: ignore[reportCallIssue]
