@@ -2,17 +2,22 @@
 
 from __future__ import annotations
 
-from datetime import datetime
+from decimal import Decimal
 from typing import TYPE_CHECKING, Any
 
 from sqlmodel import Session, SQLModel, desc, select
 
 from llmling_agent.log import get_logger
-from llmling_agent.messaging.messages import ChatMessage, TokenCost
+from llmling_agent.messaging.messages import TokenCost
+from llmling_agent.utils.now import get_now
 from llmling_agent.utils.parse_time import parse_time_period
 from llmling_agent_storage.base import StorageProvider
-from llmling_agent_storage.models import ConversationData, QueryFilters, StatsFilters
-from llmling_agent_storage.sql_provider.models import Conversation, Message
+from llmling_agent_storage.models import QueryFilters
+from llmling_agent_storage.sql_provider.models import (
+    CommandHistory,
+    Conversation,
+    Message,
+)
 from llmling_agent_storage.sql_provider.utils import (
     build_message_query,
     format_conversation,
@@ -24,13 +29,16 @@ from llmling_agent_storage.sql_provider.utils import (
 
 if TYPE_CHECKING:
     from collections.abc import Sequence
+    from datetime import datetime
 
     from sqlalchemy import Engine
 
     from llmling_agent.common_types import JsonValue
-    from llmling_agent.models.session import SessionQuery
-    from llmling_agent.models.storage import SQLStorageConfig
-    from llmling_agent.models.tools import ToolCallInfo
+    from llmling_agent.messaging.messages import ChatMessage
+    from llmling_agent.tools import ToolCallInfo
+    from llmling_agent_config.session import SessionQuery
+    from llmling_agent_config.storage import SQLStorageConfig
+    from llmling_agent_storage.models import ConversationData, StatsFilters
 
 
 logger = get_logger(__name__)
@@ -113,6 +121,7 @@ class SQLModelProvider(StorageProvider[Message]):
         self,
         *,
         conversation_id: str,
+        message_id: str,
         content: str,
         role: str,
         name: str | None = None,
@@ -129,6 +138,7 @@ class SQLModelProvider(StorageProvider[Message]):
         with Session(self.engine) as session:
             msg = Message(
                 conversation_id=conversation_id,
+                id=message_id,
                 content=content,
                 role=role,
                 name=name,
@@ -136,14 +146,14 @@ class SQLModelProvider(StorageProvider[Message]):
                 model_provider=provider,
                 model_name=model_name,
                 response_time=response_time,
-                total_tokens=cost_info.token_usage["total"] if cost_info else None,
-                prompt_tokens=cost_info.token_usage["prompt"] if cost_info else None,
-                completion_tokens=cost_info.token_usage["completion"]
+                total_tokens=cost_info.token_usage.get("total") if cost_info else None,
+                input_tokens=cost_info.token_usage.get("prompt") if cost_info else None,
+                output_tokens=cost_info.token_usage.get("completion")
                 if cost_info
                 else None,
-                cost=cost_info.total_cost if cost_info else None,
+                cost=float(cost_info.total_cost) if cost_info else None,
                 forwarded_from=forwarded_from,
-                timestamp=datetime.now(),
+                timestamp=get_now(),
             )
             session.add(msg)
             session.commit()
@@ -162,7 +172,7 @@ class SQLModelProvider(StorageProvider[Message]):
             conversation = Conversation(
                 id=conversation_id,
                 agent_name=node_name,
-                start_time=start_time or datetime.now(),
+                start_time=start_time or get_now(),
             )
             session.add(conversation)
             session.commit()
@@ -200,8 +210,6 @@ class SQLModelProvider(StorageProvider[Message]):
         metadata: dict[str, JsonValue] | None = None,
     ):
         """Log command to database."""
-        from llmling_agent_storage.sql_provider.models import CommandHistory
-
         with Session(self.engine) as session:
             history = CommandHistory(
                 session_id=session_id,
@@ -228,7 +236,7 @@ class SQLModelProvider(StorageProvider[Message]):
         """Get filtered conversations with formatted output."""
         # Convert period to since if provided
         if period:
-            since = datetime.now() - parse_time_period(period)
+            since = get_now() - parse_time_period(period)
 
         # Create filters
         filters = QueryFilters(
@@ -257,8 +265,6 @@ class SQLModelProvider(StorageProvider[Message]):
         current_session_only: bool = False,
     ) -> list[str]:
         """Get command history from database."""
-        from llmling_agent_storage.sql_provider.models import CommandHistory
-
         with Session(self.engine) as session:
             query = select(CommandHistory)
             if current_session_only:
@@ -333,8 +339,8 @@ class SQLModelProvider(StorageProvider[Message]):
                     Conversation.agent_name,
                     Message.timestamp,
                     Message.total_tokens,
-                    Message.prompt_tokens,
-                    Message.completion_tokens,
+                    Message.input_tokens,
+                    Message.output_tokens,
                 )
                 .join(Conversation, Message.conversation_id == Conversation.id)
                 .where(Message.timestamp > filters.cutoff)
@@ -355,7 +361,7 @@ class SQLModelProvider(StorageProvider[Message]):
                             "prompt": prompt or 0,
                             "completion": completion or 0,
                         },
-                        total_cost=0.0,  # We don't store this in DB
+                        total_cost=Decimal(0),  # We don't store this in DB
                     )
                     if total or prompt or completion
                     else None,
