@@ -19,6 +19,7 @@ from llmling_agent.utils.now import get_now
 if TYPE_CHECKING:
     from llmling_agent.agent import AnyAgent
     from llmling_agent.common_types import StrPath
+    from llmling_agent_config.mcp_server import MCPServerConfig
 
 logger = get_logger(__name__)
 
@@ -74,8 +75,6 @@ async def list_available_agents(  # noqa: D417
     """
     from pydantic_ai.tools import RunContext
 
-    from llmling_agent_providers.base import AgentLLMProvider
-
     if isinstance(ctx, RunContext):
         ctx = ctx.deps
     if not ctx.pool:
@@ -92,7 +91,7 @@ async def list_available_agents(  # noqa: D417
         lines.extend([
             f"name: {name}",
             f"description: {agent.description or 'No description'}",
-            f"type: {'ai' if isinstance(agent.provider, AgentLLMProvider) else 'human'}",
+            f"model: {agent.model_name}",
             "---",
         ])
 
@@ -186,6 +185,7 @@ async def spawn_delegate[TDeps](
     Creates an ephemeral agent that will execute the task and clean up automatically
     Optionally connects back to receive results.
     """
+    from pydantic_ai.exceptions import ModelRetry
     from pydantic_ai.tools import RunContext
 
     from llmling_agent import Agent
@@ -208,8 +208,11 @@ async def spawn_delegate[TDeps](
     if connect_back:
         assert ctx.agent
         ctx.agent.connect_to(agent)
-
-    await agent.run(task)
+    try:
+        await agent.run(task)
+    except Exception as e:
+        msg = f"Failed to spawn delegate {name}: {e}"
+        raise ModelRetry(msg) from e
     return f"Spawned delegate {name} for task"
 
 
@@ -348,26 +351,30 @@ async def add_agent(  # noqa: D417
         name: Name for the new agent
         system_prompt: System prompt defining agent's role/behavior
         model: Optional model override (uses default if not specified)
-        tools: Names of tools to enable for this agent
+        tools: Imort paths of the tools the agent should have, if any.
         session: Session ID to recover conversation state from
         result_type: Name of response type from manifest (for structured output)
 
     Returns:
         Confirmation message about the created agent
     """
+    from pydantic_ai.exceptions import ModelRetry
     from pydantic_ai.tools import RunContext
 
     if isinstance(ctx, RunContext):
         ctx = ctx.deps
     assert ctx.pool, "No agent pool available"
-    agent: AnyAgent[Any, Any] = await ctx.pool.add_agent(
-        name=name,
-        system_prompt=system_prompt,
-        model=model,
-        tools=tools,
-        result_type=result_type,
-        session=session,
-    )
+    try:
+        agent: AnyAgent[Any, Any] = await ctx.pool.add_agent(
+            name=name,
+            system_prompt=system_prompt,
+            model=model,
+            tools=tools,
+            result_type=result_type,
+            session=session,
+        )
+    except ValueError as e:  # for wrong tool imports
+        raise ModelRetry(message=f"Error creating agent: {e}") from None
     return f"Created agent {agent.name} using model {agent.model_name}"
 
 
@@ -426,13 +433,18 @@ async def ask_agent(  # noqa: D417
     Returns:
         The agent's response
     """
+    from pydantic_ai.exceptions import ModelRetry
     from pydantic_ai.tools import RunContext
 
     if isinstance(ctx, RunContext):
         ctx = ctx.deps
     assert ctx.pool, "No agent pool available"
     agent = ctx.pool.get_agent(agent_name)
-    result = await agent.run(message, model=model, store_history=store_history)
+    try:
+        result = await agent.run(message, model=model, store_history=store_history)
+    except Exception as e:
+        msg = f"Failed to ask agent {agent_name}: {e}"
+        raise ModelRetry(msg) from e
     return str(result.content)
 
 
@@ -825,3 +837,100 @@ async def ask_user(  # noqa: D417
             return f"Error: {message}"
         case _:
             return "Unknown error occurred"
+
+
+async def add_local_mcp_server(  # noqa: D417
+    ctx: AgentContext,
+    name: str,
+    command: str,
+    args: list[str] | None = None,
+    env_vars: dict[str, str] | None = None,
+) -> str:
+    """Add a local MCP server via stdio transport.
+
+    Args:
+        name: Unique name for the MCP server
+        command: Command to execute for the server
+        args: Command arguments
+        env_vars: Environment variables to pass to the server
+
+    Returns:
+        Confirmation message about the added server
+    """
+    from pydantic_ai.tools import RunContext
+
+    from llmling_agent_config.mcp_server import StdioMCPServerConfig
+
+    if isinstance(ctx, RunContext):
+        ctx = ctx.deps
+
+    env = env_vars or {}
+    config = StdioMCPServerConfig(name=name, command=command, args=args or [], env=env)
+    ctx.agent.mcp.add_server_config(config)
+    await ctx.agent.mcp.setup_server(config)
+
+    return f"Added local MCP server {name!r} with command: {command}"
+
+
+async def add_remote_mcp_server(  # noqa: D417
+    ctx: AgentContext,
+    name: str,
+    url: str,
+    transport: Literal["sse", "streamable-http"] = "streamable-http",
+) -> str:
+    """Add a remote MCP server via HTTP-based transport.
+
+    Args:
+        name: Unique name for the MCP server
+        url: Server URL endpoint
+        transport: HTTP transport type to use (http is preferred)
+
+    Returns:
+        Confirmation message about the added server
+    """
+    from pydantic_ai.tools import RunContext
+
+    from llmling_agent_config.mcp_server import (
+        SSEMCPServerConfig,
+        StreamableHTTPMCPServerConfig,
+    )
+
+    if isinstance(ctx, RunContext):
+        ctx = ctx.deps
+
+    match transport:
+        case "sse":
+            config: MCPServerConfig = SSEMCPServerConfig(name=name, url=url)
+        case "streamable-http":
+            config = StreamableHTTPMCPServerConfig(name=name, url=url)
+
+    ctx.agent.mcp.add_server_config(config)
+    await ctx.agent.mcp.setup_server(config)
+
+    return f"Added remote MCP server '{name}' at {url} using {transport} transport"
+
+
+if __name__ == "__main__":
+    # import logging
+    from llmling_agent import AgentPool, Capabilities
+
+    user_prompt = """Add a stdio MCP server:
+// 	"command": "npx",
+// 	"args": ["mcp-graphql"],
+// 	"env": { "ENDPOINT": "https://diego.one/graphql" }
+
+."""
+
+    async def main():
+        async with AgentPool() as pool:
+            caps = Capabilities(can_add_mcp_servers=True)
+            agent = await pool.add_agent(
+                "X", capabilities=caps, model="openai:gpt-5-nano"
+            )
+            agent.tool_used.connect(print)
+            result = await agent.run(user_prompt)
+            print(result)
+            result = await agent.run("Which tools does it have?")
+            print(result)
+
+    asyncio.run(main())
